/*
  Rhino test framework

   Copyright (C) 2011, 2012, 2013, 2014  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.RhinoTests;

import java.util.List;

import javax.script.ScriptEngineFactory;
import javax.script.ScriptEngineManager;



/**
 * Basic ScriptEngineFactory test suite.
 *
 * @author Pavel Tisnovsky
 */
public class ScriptEngineFactoryTest extends BaseRhinoTest {
    ScriptEngineManager engineManager;

    @Override
    protected void setUp(String[] args) {
        this.engineManager = new ScriptEngineManager();
    }

    @Override
    protected void tearDown() {
        // empty block
        return;
    }

    /**
     * Test if ScriptEngineManager.getEngineFactories() does not returns null.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineManagerGetEngineFactoriesNotNull() throws Exception {
        List<ScriptEngineFactory> engineFactories = this.engineManager.getEngineFactories();
        assertNotNull(engineFactories, "scriptManager.getEngineFactories() returns null");
    }

    /**
     * Test if at least one factory is returned by ScriptEngineManager.getEngineFactories().
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineManagerGetEngineFactoriesNotEmpty() throws Exception {
        List<ScriptEngineFactory> engineFactories = this.engineManager.getEngineFactories();
        assertTrue(!engineFactories.isEmpty(), "scriptManager.getEngineFactories() returns empty string");
    }

    /**
     * This tests simply lists informations about all available script engine factories.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineFactoryList() throws Exception {
        for (ScriptEngineFactory factory : this.engineManager.getEngineFactories()) {
            String engineName = factory.getEngineName();
            String engineVersion = factory.getEngineVersion();
            String languageName = factory.getLanguageName();
            String languageVersion = factory.getLanguageVersion();

            System.out.printf("\tScript Engine: %s (%s)%n", engineName, engineVersion);
            for (String name : factory.getNames()) {
                System.out.printf("\tEngine Alias: %s%n", name);
            }
            System.out.printf("\tLanguage: %s (%s)%n", languageName, languageVersion);
        }
    }

    /**
     * Check if basic info returned by script engine factories does not contain any null values.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineFactoryListNotNullAttributes() throws Exception {
        for (ScriptEngineFactory factory : this.engineManager.getEngineFactories()) {
            String engineName = factory.getEngineName();
            String engineVersion = factory.getEngineVersion();
            String languageName = factory.getLanguageName();
            String languageVersion = factory.getLanguageVersion();

            assertNotNull(engineName, "factory.getEngineName() is null");
            assertNotNull(engineVersion, "factory.getEngineVersion() is null");
            assertNotNull(languageName, "factory.getLanguageName() is null");
            assertNotNull(languageVersion, "factory.getLanguageVersion() is null");
        }
    }

    /**
     * Check if basic info returned by script engine factories does not contain any empty values.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineFactoryListNotEmptyAttributes() throws Exception {
        for (ScriptEngineFactory factory : this.engineManager.getEngineFactories()) {
            String engineName = factory.getEngineName();
            String engineVersion = factory.getEngineVersion();
            String languageName = factory.getLanguageName();
            String languageVersion = factory.getLanguageVersion();

            assertNotEmpty(engineName.trim(), "factory.getEngineName() is empty");
            assertNotEmpty(engineVersion.trim(), "factory.getEngineVersion() is empty");
            assertNotEmpty(languageName.trim(), "factory.getLanguageName() is empty");
            assertNotEmpty(languageVersion.trim(), "factory.getLanguageVersion() is empty");
        }
    }

    /**
     * Check if script engine factory aliases are not null.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineFactoryListNotNullAliases() throws Exception {
        for (ScriptEngineFactory factory : this.engineManager.getEngineFactories()) {
            for (String name : factory.getNames()) {
                assertNotNull(name, "alias is null");
            }
        }
    }

    /**
     * Check if script engine factory aliases are not empty.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineFactoryListNotEmptyAliases() throws Exception {
        for (ScriptEngineFactory factory : this.engineManager.getEngineFactories()) {
            for (String name : factory.getNames()) {
                assertNotEmpty(name.trim(), "alias is empty string");
            }
        }
    }

    /**
     * Check if script engine factory names contain given name.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void checkIfEngineFactoriesContainName(String name) throws Exception {
        for (ScriptEngineFactory factory : this.engineManager.getEngineFactories()) {
            for (String engineName : factory.getNames()) {
                if (engineName.equals(name)) {
                    return;
                }
            }
        }
        throw new Exception("JavaScript script engine factory not found for name '" + name + "'!");
    }

    /**
     * Check if Rhino engine can be found by using name "javascript"
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineFactoriesContainjavascript() throws Exception {
        checkIfEngineFactoriesContainName(Constants.EngineNames.ENGINE_NAME_javascript);
    }

    /**
     * Check if Rhino engine can be found by using name "JavaScript"
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineFactoriesContainJavaScript() throws Exception {
        checkIfEngineFactoriesContainName(Constants.EngineNames.ENGINE_NAME_JavaScript);
    }

    /**
     * Check if Rhino engine can be found by using name "ecmascript"
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineFactoriesContainecmascript() throws Exception {
        checkIfEngineFactoriesContainName(Constants.EngineNames.ENGINE_NAME_ecmascript);
    }

    /**
     * Check if Rhino engine can be found by using name "ECMAScript"
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineFactoriesContainECMAScript() throws Exception {
        checkIfEngineFactoriesContainName(Constants.EngineNames.ENGINE_NAME_ECMAScript);
    }

    /**
     * Check if Rhino engine can be found by using name "rhino"
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineFactoriesContainRhino() throws Exception {
        checkIfEngineFactoriesContainName(Constants.EngineNames.ENGINE_NAME_rhino);
    }

    /**
     * Check if Rhino engine can be found by using name "js"
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineFactoriesContainJs() throws Exception {
        checkIfEngineFactoriesContainName(Constants.EngineNames.ENGINE_NAME_js);
    }

    /**
     * Check if js extension is supported by at least one script engine factory.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineFactoriesExtensiosContainJs() throws Exception {
        for (ScriptEngineFactory factory : this.engineManager.getEngineFactories()) {
            boolean found = false;
            for (String engineName : factory.getNames()) {
                if (engineName.equals("javascript")) {
                    found = true;
                }
            }
            if (found) {
                for (String extension : factory.getExtensions()) {
                    if (extension.equals("js")) {
                        return;
                    }
                }
            }
        }
        throw new Exception("JavaScript script engine factory does not support .js extension!");
    }

    /**
     * Check if certain MIME type is supported by at least one script engine factory.
     * @param name
     *
     * @throws Exception exception thrown in case this test fails
     */
    private void checkIfEngineFactoriesMimeTypesContainName(String name) throws Exception {
        for (ScriptEngineFactory factory : this.engineManager.getEngineFactories()) {
            boolean found = false;
            for (String engineName : factory.getNames()) {
                if (engineName.equals("javascript")) {
                    found = true;
                }
            }
            if (found) {
                for (String mimeType : factory.getMimeTypes()) {
                    if (mimeType.contains(name)) {
                        return;
                    }
                }
            }
        }
        throw new Exception("JavaScript script engine factory does not support proper MIME type '" + name + "'!");
    }

    /**
     * Check if supported MIME types list contains "text/javascript".
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineFactoriesMimeTypesContainsJavascript() throws Exception {
        checkIfEngineFactoriesMimeTypesContainName(Constants.MimeTypes.MIME_TYPE_TEXT_JAVASCRIPT);
    }

    /**
     * Check if supported MIME types list contains "text/ecmascript".
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineFactoriesMimeTypesContainsEcmaScript() throws Exception {
        checkIfEngineFactoriesMimeTypesContainName(Constants.MimeTypes.MIME_TYPE_TEXT_ECMASCRIPT);
    }

    /**
     * Check if supported MIME types list contains "application/javascript".
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineFactoriesMimeTypesContainsApplicationJavascript() throws Exception {
        checkIfEngineFactoriesMimeTypesContainName(Constants.MimeTypes.MIME_TYPE_APPLICATION_JAVASCRIPT);
    }

    /**
     * Check if supported MIME types list contains "application/ecmascript".
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineFactoriesMimeTypesContainsApplicationEcmaScript() throws Exception {
        checkIfEngineFactoriesMimeTypesContainName(Constants.MimeTypes.MIME_TYPE_APPLICATION_ECMASCRIPT);
    }

    /**
     * Check if method getMethodCallSyntax() does not return null.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineFactoryGetMethodCallSyntaxNotNull() throws Exception {
        for (ScriptEngineFactory factory : this.engineManager.getEngineFactories()) {
            String firstMethodCall = factory.getMethodCallSyntax("String", "length");
            String secondMethodCall = factory.getMethodCallSyntax("Integer", "compareTo", "x");
            assertNotNull(firstMethodCall, "factory.getMethodCallSyntax() returns null");
            assertNotNull(secondMethodCall, "factory.getMethodCallSyntax() returns null");
            System.out.println("\t" + firstMethodCall);
            System.out.println("\t" + secondMethodCall);
        }
    }

    /**
     * Check if method getMethodCallSyntax() does not return empty string.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineFactoryGetMethodCallSyntaxNotEmpty() throws Exception {
        for (ScriptEngineFactory factory : this.engineManager.getEngineFactories()) {
            String firstMethodCall = factory.getMethodCallSyntax("String", "length");
            String secondMethodCall = factory.getMethodCallSyntax("Integer", "compareTo", "x");
            assertNotEmpty(firstMethodCall, "factory.getMethodCallSyntax() returns null");
            assertNotEmpty(secondMethodCall, "factory.getMethodCallSyntax() returns null");
            System.out.println("\t" + firstMethodCall);
            System.out.println("\t" + secondMethodCall);
        }
    }

    /**
     * Check if method getOutputStatement() does not return null.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineFactoryGetOutputStatementNotNull() throws Exception {
        for (ScriptEngineFactory factory : this.engineManager.getEngineFactories()) {
            String outputStatement = factory.getOutputStatement("foo bar");
            assertNotNull(outputStatement, "factory.getOutputStatement() returns null");
            System.out.println("\t" + outputStatement);
        }
    }

    /**
     * Check if method getOutputStatement() does not return empty string.
     *
     * @throws Exception exception thrown in case this test fails
     */
    protected void testScriptEngineFactoryGetOutputStatementNotEmpty() throws Exception {
        for (ScriptEngineFactory factory : this.engineManager.getEngineFactories()) {
            String outputStatement = factory.getOutputStatement("foo bar");
            assertNotEmpty(outputStatement.trim(), "factory.getOutputStatement() returns null");
            System.out.println("\t" + outputStatement);
        }
    }

    /**
     * Entry point to this test case.
     *
     * @param args parameters passed from command line
     */
    public static void main(String[] args) {
        new ScriptEngineFactoryTest().doTests(args);
    }

}
